#include "truss.hpp"
#include "intvector.hpp"
#include "rvector.hpp"
#include "rmatrix.hpp"
#include <fstream>

using namespace std;
using namespace cxsc;


void inputtruss(const char* filename, int& n, int& p, int& m, rvector& x, rvector& y, intvector& a, intvector& b) {
	/* Open the input file */
	fstream file;
	file.open(filename,fstream::in);
	
	/* Read number of free and fixed joints */
	file >> n;
	file >> p;

	/* Allocate memory for the joint coordinates */
	rvector xx(n+p);
	rvector yy(n+p);

	/* Read joint coordinates */
	for (int i=1; i <= n+p; i++) {
		file >> xx[i];
		file >> yy[i];
	}
	x = xx;
    y = yy;

	/* Read number of members */
	file >> m;

	/* Allocate memory for the start and end joints of all members */
	intvector aa(m);
	intvector bb(m);

	/* Read the indices of the start and end joints of each member */
	for (int i=1; i <= m; i++) {
		file >> aa[i];
		file >> bb[i];
	}
	a = aa;
    b = bb;
	
	/* Close the input file */
	file.close();
}


int membernumber(int m, intvector a, intvector b, int i, int j) {
	int k = 0;
	for (int l=1; l <= m; l++) {
		if ((a[l] == i && b[l] == j) || (a[l] == j && b[l] == i)) {
			k = l;
			break;
		}
	}
	return k;
}


rmatrix stiffness() {
	/* Number of free joints */
	int n;
	
	/* Number of fixed joints */
	int p;

	/* Number of members */
	int m;

	/* Joint coordinates */
	rvector x;
	rvector y;

	/* Indices of the start and end joints of each member */
	intvector a;
	intvector b;

	/* Read truss data from an input file */
	inputtruss("truss.txt", n, p, m, x, y, a, b);

	/* Equilibrium matrix and its transposed */
	rmatrix E(2*n,m);
	rmatrix ET(m,2*n);
	rmatrix L(m,m);

	for (int i=1; i <= 2*n; i++) {
		for (int j=1; j <= m; j++) {
			E[i][j] = 0;
			ET[j][i] = 0;
		}
	}

	for (int i=1; i <= m; i++) {
		for (int j=1; j <= m; j++) {
			L[i][j] = 0;
		}
	}

	int k;
	real s;
	for (int i=1; i <= n; i++) {
		for (int j=1; j <= n+p; j++) {
			k = membernumber(m,a,b,i,j);
			s = 1;
			if (j < i) {
				s = -1;
			}
			if ( k > 0 ) {
				real l = (x[j] - x[i]) * (x[j] - x[i]) + (y[j] - y[i]) * (y[j] - y[i]);
				l = sqrt(l);
				L[k][k] = l;
				E[2*i-1][k] = s * (x[j] - x[i]) / l;
				E[2*i][k] = s * (y[j] - x[i]) / l;
				ET[k][2*i-1] = E[2*i-1][k];
				ET[k][2*i] = E[2*i][k];
			}
		}
	}
	
	rmatrix A = 7500 * E * L * ET;
	return A;
}